package com.ekexiu.project.qxweb.servlet;

import com.ekexiu.project.qxweb.monitor.MonitorLog;
import org.jfw.util.DateUtil;
import org.jfw.util.context.JfwAppContext;
import org.jfw.util.io.IoUtil;
import org.jfw.util.json.JsonService;
import org.jfw.util.reflect.TypeReference;

import static java.lang.Thread.sleep;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.sql.DataSource;
import java.io.IOException;
import java.io.InputStream;
import java.lang.reflect.Type;
import java.net.ServerSocket;
import java.net.Socket;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CopyOnWriteArrayList;
import java.util.concurrent.atomic.AtomicBoolean;

/**
 * Created by TT on 2019/4/24.
 */
public class MonitorServlet extends HttpServlet {
    private static final long serialVersionUID = -547145402837294129L;
    private DataSource dataSource;
    private AtomicBoolean running = new AtomicBoolean(false);
    private ListenThread listenThread;
    private List<DataThread> dataThreadList = new CopyOnWriteArrayList<DataThread>();
    public static Type DATA_TYPE = new TypeReference<Map<String, Object>>() {
    }.getType();

    //private static Gson gson = new GsonBuilder().registerTypeAdapter();

    @Override
    public void init() throws ServletException {
        dataSource = JfwAppContext.getDataSource();
        running.set(true);
        try {
            ServerSocket serverSocket = new ServerSocket(5566);
            listenThread = new ListenThread(serverSocket);
            listenThread.start();
        } catch (IOException e) {
            e.printStackTrace();
        }
    }

    @Override
    public void destroy() {
        this.running.set(false);
        if (listenThread != null) {
            listenThread.interrupt();

            for (; ; ) {
                if (this.dataThreadList.size() > 0) {
                    for (Thread thread : this.dataThreadList) {
                        thread.interrupt();
                    }
                    try {
                        sleep(500);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                } else {
                    break;
                }
            }

        }
    }

    class DataThread extends Thread {
        private Socket socket;

        public DataThread(Socket socket) {
            this.socket = socket;
        }

        @Override
        public void run() {
            int idx, len, ret, offset;
            InputStream inputStream = null;
            while (running.get()) {
                try {
                    if (inputStream == null) {
                        inputStream = socket.getInputStream();
                    }
                    String createTime = DateUtil.formatDateTime(System.currentTimeMillis());
                    byte[] head = new byte[14];
                    idx = 0;
                    len = 14;
                    while (len > 0) {
                        ret = inputStream.read(head, idx, len);
                        if (ret >= 0) {
                            idx += ret;
                            len -= ret;
                        } else {
                            throw new IOException("a");
                        }
                    }

                    String hexHead = bytesToHexString(head);
                    assert hexHead != null;
                    String mac = hexHead.substring(12, 24);

                    byte[] tail = IoUtil.readStream(inputStream, true);
                    byte[] body = new byte[tail.length - 3];
                    System.arraycopy(tail, 0, body, 0, tail.length - 3);
                    String hexBody = bytesToHexString(body);

                    String data = hexStringToString(hexBody);
                    System.out.println(data);

                    MonitorLog monitorLog;
                    try {
                        monitorLog = JsonService.fromJson(data, MonitorLog.class);
                    } catch (Exception e) {
                        return;
                    }

                    Map<String, Object> map = JsonService.fromJson(data, DATA_TYPE);
                    System.out.println(map);

                    monitorLog.setCreateTime(createTime);
                    monitorLog.setSite(mac);
                    System.out.println(monitorLog.toString());
                    //try {
                    //    Connection con = dataSource.getConnection();
                    //    try {
                    //        //PreparedStatement ps = con.prepareStatement("INSERT INTO ENV_DATA (ITEM_ID,ITEM_VALUE,ITEM_TIME,CREATE_TIME) VALUES (?,?,?,?)");
                    //        //ps.executeUpdate();
                    //        con.commit();
                    //        //envDataService.setLastData(list);
                    //    } catch (SQLException e) {
                    //        con.rollback();
                    //    } finally {
                    //        con.close();
                    //    }
                    //} catch (Throwable thr) {
                    //
                    //}
                } catch (IOException e) {
                    break;
                }
            }
        }
    }

    class ListenThread extends Thread {
        private ServerSocket serverSocket;


        public ListenThread(ServerSocket serverSocket) {
            this.serverSocket = serverSocket;
        }

        @Override
        public void run() {
            while (running.get()) {
                try {
                    Socket socket = serverSocket.accept();
                    DataThread dataThread = new DataThread(socket);
                    dataThreadList.add(dataThread);
                    dataThread.start();
                } catch (Exception e) {
                }
            }
        }
    }

    public static String hexStringToString(String s) {
        if (s == null || s.equals("")) {
            return null;
        }
        s = s.replace(" ", "");
        byte[] baKeyword = new byte[s.length() / 2];
        for (int i = 0; i < baKeyword.length; i++) {
            try {
                baKeyword[i] = (byte) (0xff & Integer.parseInt(s.substring(i * 2, i * 2 + 2), 16));
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        try {
            s = new String(baKeyword, "UTF-8");
        } catch (Exception e1) {
            e1.printStackTrace();
        }
        return s;
    }

    public static String bytesToHexString(byte[] src) {
        StringBuilder stringBuilder = new StringBuilder("");
        if (src == null || src.length <= 0) {
            return null;
        }
        for (int i = 0; i < src.length; i++) {
            int v = src[i] & 0xFF;
            String hv = Integer.toHexString(v);
            if (hv.length() < 2) {
                stringBuilder.append(0);
            }
            stringBuilder.append(hv);
        }
        return stringBuilder.toString();
    }

    public static void headtoString(byte[] head, int idx, int len) {
        int rlen = idx + len;
        System.out.print("[");
        for (int i = idx; i < rlen; ++i) {
            System.out.print(head[i]);
            System.out.print(",");
        }
        System.out.println("]");
    }
}
