package com.ekexiu.project.qxweb.system.service;


import com.ekexiu.project.qxweb.right.po.UserSite;
import com.ekexiu.project.qxweb.system.dao.UserDao;
import com.ekexiu.project.qxweb.system.po.User;
import com.ekexiu.project.qxweb.system.vo.SessionUser;
import org.jfw.apt.annotation.Autowrie;
import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.web.annotation.LoginUser;
import org.jfw.apt.web.annotation.Path;
import org.jfw.apt.web.annotation.method.InvalidSession;
import org.jfw.apt.web.annotation.method.SetSession;
import org.jfw.apt.web.annotation.operate.Get;
import org.jfw.apt.web.annotation.operate.Post;
import org.jfw.apt.web.annotation.param.JdbcConn;
import org.jfw.apt.web.annotation.param.SessionVal;
import org.jfw.util.PageQueryResult;
import org.jfw.util.StringUtil;
import org.jfw.util.exception.JfwBaseException;

import java.sql.Connection;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

/**
 * Created by TT on 2019/4/9.
 */
@Path("/sys")
public class SysService {

    public static final String DEFAULT_PW_STR = StringUtil.md5("123456");

    @Autowrie
    private UserDao userDao;

    public UserDao getUserDao() {
        return userDao;
    }

    public void setUserDao(UserDao userDao) {
        this.userDao = userDao;
    }

    @SetSession("JFW_SESSION_LOGIN_USER=result")
    @Path("/login")
    @Post
    public SessionUser login(@JdbcConn Connection con, String account, String pw, String vc, @Nullable @SessionVal(value = "PIC_LOGIN",remove = true)String code, @Nullable @SessionVal(value = "TIMEOUT_PIC_LOGIN",remove = true)Long timeout)throws SQLException, JfwBaseException {
        if (code == null || timeout == null) {
            throw new JfwBaseException(-60001, "not found sessionVal");
        }
        if (!vc.equals(code)) {
            throw new JfwBaseException(-60003, "vc not right");
        }
        if (System.currentTimeMillis() > timeout) {
            throw new JfwBaseException(-60002, "timeout");
        }
        User user = userDao.login(con, account, StringUtil.md5(pw));
        if (user != null) {
            if(!user.isActive()){
                throw new JfwBaseException(-60005,"user is disabled");
            }
            return makeSessionUser(user);
        }
        return null;
    }

    @Get
    @Path("/user")
    public SessionUser get(@LoginUser SessionUser user) {
        return user;
    }

    @Get
    @Path("/logout")
    @InvalidSession
    public void logout() {
    }

    @Post
    @Path("/insert")
    public String insert(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
        String id = StringUtil.buildUUID();
        user.setId(id);
        user.setActive(true);
        user.setPasswd(DEFAULT_PW_STR);
        user.setCreator(sessionUser.getId());
        user.setModifier(sessionUser.getId());
        this.userDao.insert(con, user);
        return id;
    }

    @Get
    @Path("/check")
    public Boolean check(@JdbcConn Connection con,String account,@Nullable String id)throws SQLException {
        return this.userDao.queryByAccount(con, account,id)==null;
    }

    @Get
    @Path("/qo")
    public SessionUser query(@JdbcConn Connection con, String id) throws SQLException {
        return makeSessionUser(this.userDao.query(con, id));
    }

    @Post
    @Path("/update")
    public void update(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, User user) throws SQLException {
        user.setModifier(sessionUser.getId());
        this.userDao.update(con, user);
    }

    @Get
    @Path("/ban")
    public void ban(@JdbcConn(true) Connection con, @LoginUser SessionUser sessionUser, String id) throws SQLException {
        this.userDao.ban(con, sessionUser.getId(), id);
    }

    @Post
    @Path("/changePw")
    public void changePw(@JdbcConn(true) Connection con, @LoginUser SessionUser u, String newPw) throws SQLException {
        this.userDao.changePw(con, StringUtil.md5(newPw), u.getId());
    }

    @Post
    @Path("/resetPw")
    public void resetPw(@JdbcConn(true) Connection con,@LoginUser SessionUser user,String id)throws SQLException {
        this.userDao.changePw(con, DEFAULT_PW_STR, id);
    }

    @Get
    @Path("/pq")
    public PageQueryResult<SessionUser> pageQuery(@JdbcConn Connection con, @Nullable Boolean active, @Nullable String account, @Nullable String name,
                                                  int pageSize, int pageNo) throws SQLException {
        PageQueryResult<User> pageQueryResult = this.userDao.pageQuery(con,active, account, name == null ? null : "%" + name + "%", pageSize, pageNo);
        List<User> users = pageQueryResult.getData();
        List<SessionUser> sessionUsers = new ArrayList<>();
        if (!users.isEmpty()) {
            for (User user : users) {
                sessionUsers.add(makeSessionUser(user));
            }
            PageQueryResult<SessionUser> ret = new PageQueryResult<>();
            ret.setPageNo(pageQueryResult.getPageNo());
            ret.setPageSize(pageQueryResult.getPageSize());
            ret.setTotal(pageQueryResult.getTotal());
            ret.setData(sessionUsers);
            return ret;
        }
        return null;
    }

    @Path("/userSite")
    @Post
    public void userSite(@JdbcConn(true) Connection con,@Nullable String[] sites,String id)throws SQLException {
        userDao.delete(con, id);
        if (sites != null && sites.length > 0) {
            UserSite[] userSites = new UserSite[sites.length];
            for (int i = 0; i < sites.length; ++i) {
                UserSite ub = new UserSite();
                ub.setUid(id);
                ub.setSite(sites[i]);
                userSites[i] = ub;
            }
            this.userDao.insert(con, userSites);
        }
    }


    private static SessionUser makeSessionUser(User user) {
        SessionUser sessionUser = new SessionUser();
        sessionUser.setAccount(user.getAccount());
        sessionUser.setName(user.getName());
        sessionUser.setComp(user.getComp());
        sessionUser.setEmail(user.getEmail());
        sessionUser.setId(user.getId());
        sessionUser.setJob(user.getJob());
        sessionUser.setPhone(user.getPhone());
        sessionUser.setRemark(user.getRemark());
        sessionUser.setActive(user.isActive());
        return sessionUser;
    }
}
